@@ -1,5 +1,42 @@
 library(data.table)
 # options(joyn.verbose = FALSE)
+x1 = data.table(id = c(1L, 1L, 2L, 3L, NA_integer_),
+                t  = c(1L, 2L, 1L, 2L, NA_integer_),
+                x  = 11:15)
+
+y1 = data.table(id = c(1,2, 4),
+                y  = c(11L, 15L, 16))
+
+
+x2 = data.table(id = c(1, 4, 2, 3, NA),
+                t  = c(1L, 2L, 1L, 2L, NA_integer_),
+                x  = c(16, 12, NA, NA, 15))
+
+
+y2 = data.table(id = c(1, 2, 5, 6, 3),
+                yd = c(1, 2, 5, 6, 3),
+                y  = c(11L, 15L, 20L, 13L, 10L),
+                x  = c(16:20))
+
+
+y3 <- data.table(id = c("c","b", "c", "a"),
+                 y  = c(11L, 15L, 18L, 20L))
+
+x3 <- data.table(id=c("c","b", "d"),
+                 v=8:10,
+                 foo=c(4,2, 7))
+
+x4 = data.table(id1 = c(1, 1, 2, 3, 3),
+                id2 = c(1, 1, 2, 3, 4),
+                t   = c(1L, 2L, 1L, 2L, NA_integer_),
+                x   = c(16, 12, NA, NA, 15))
+
+
+y4 = data.table(id  = c(1, 2, 5, 6, 3),
+                id2 = c(1, 1, 2, 3, 4),
+                y   = c(11L, 15L, 20L, 13L, 10L),
+                x   = c(16:20))
+
 
 test_that("slect by vars when no specified", {
   expect_equal(merge(x1, y1),
@@ -514,4 +551,9 @@ test_that("no matching obs", {
 
 })
 
+test_that("convert to data.table", {
+  xx1 <- as.data.frame(x1)
+  expect_equal(merge(xx1, y1), merge(x1, y1))
+})
+
 
